/**
 * @file IA.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Fonctions de l'Intelligence Artificielle.
 * L'intelligence Artificielle est basée sur l'algorithme du min-max et de l'alpha-béta
 *
 *	Un grand merci à Bruce Moreland pour ses articles très intéressants
 *	http://www.seanet.com/~brucemo/topics/topics.htm
 *	
 */


#include "IA.h"


PPiece* piece_vide;

struct CoupAJouer
{
	unsigned char pos_depart;
	unsigned char pos_arrivee;
};

struct InfoMinMax
{
	PPiece** echiquier;
	unsigned char profondeur;
	unsigned char couleur;
	CoupAJouer* meilleur_coup;
	bool* IA_doit_finir_immediatement;
	bool* faible_priorite;
	int compteur_pour_la_faible_priorite;
};


/*	Fonction la plus importante de l'Intelligence Artificielle.
	Elle retourne un score qui est l'évaluation statique de l'échiquier
	Si le score est positif l'avantage est du côté de la couleur passée en paramètre
	si le score est nul il y a égalité et sinon
	si le score est négatif l'avantage n'est pas du côté de la couleur passée en paramètre
*/
int evaluation_echiquier(const InfoMinMax* infos)
{
	int eval=0;
	unsigned char couleur=infos->couleur;
	PPiece** echiquier=infos->echiquier;


	for(int i=0;i<64;i++)
	{
		if (echiquier[i]->get_couleur()==couleur)
			eval+=echiquier[i]->get_valeur(); 
		else 
			eval-=echiquier[i]->get_valeur();
	}
	
	
	return eval;
}





int AlphaBeta(InfoMinMax* infos, unsigned char profondeur, int alpha, int beta)
{
	unsigned char* pdp=NULL;//pour récupérer les déplacements possibles
	unsigned char possibilites_de_deplacement[32];// pour sauver les déplacements possibles
	int j;// j est initialisée un peu plus bas, pas de soucis
	int evaluation=-INFINI;// l'évaluation donnée par la fonction d'évaluation (-INFINI car au début ça ne peut pas être pire)
	PPiece* piece;// piece de sauvegarde

	// on est dans un thread et la demande d'arrêt a été demandée donc on quitte
	// la valeur de retour n'a donc aucune importance
	if (*(infos->IA_doit_finir_immediatement))
	{
		if (infos->meilleur_coup->pos_depart<65 && infos->meilleur_coup->pos_arrivee<65)
			return 0;
	}
	
	if (*(infos->faible_priorite))
	{
		infos->compteur_pour_la_faible_priorite++;
		if (infos->compteur_pour_la_faible_priorite==10000)
		{
			SDL_Delay(5);
			infos->compteur_pour_la_faible_priorite=0;
		}
	}
	
	if (profondeur==0)
		return evaluation_echiquier(infos);
	
	// pour chacune des pièces du jeu
	for(int i=0;i<64;i++)
	{
		// si on a sélectionné une pièce qui ne nous appartient pas on passe à la suivante
		if (infos->echiquier[i]->get_couleur()!=infos->couleur)
			continue;
		
		// on récupère ses possibilités de déplacement pour la pièce sélectionnée
		pdp=infos->echiquier[i]->possibilites_de_deplacement(true);
		for(j=0;j<32;j++)
			possibilites_de_deplacement[j]=pdp[j];
		
		j=0;
		// Tant qu'il y a un déplacement possible
		while (possibilites_de_deplacement[j]<65)
		{
			/*	on va déplacer notre pièce mais si à l'endroit du déplacement
				il y a une autre pièce on fait attention de s'en rappeler pour
				qu'au moment où on voudra "défaire" le coup on puisse la remettre
			*/
			piece=infos->echiquier[possibilites_de_deplacement[j]];
			infos->echiquier[i]->deplacer_rapidement(possibilites_de_deplacement[j]);
			infos->echiquier[i]=piece_vide;
			
			if (infos->couleur==BLANC)
				infos->couleur=NOIR;
			else
				infos->couleur=BLANC;
			
			
			// on change le signe pour minimiser ou maximiser
			evaluation=-AlphaBeta(infos,profondeur-1,-beta,-alpha);
			
			if (infos->couleur==BLANC)
				infos->couleur=NOIR;
			else
				infos->couleur=BLANC;

			// on "défait" le coup qui vient d'être joué
			infos->echiquier[possibilites_de_deplacement[j]]->deplacer_rapidement(i);
			// on remet la pièce qui avait été là au début ie: avant le déplacement
			infos->echiquier[possibilites_de_deplacement[j]]=piece;
		
			
			//  coupure alpha-béta
			if (evaluation>=beta)
				return beta;
			
			if (evaluation>alpha)
			{
				alpha=evaluation;
				// si on est remonté tout en haut de l'arbre on sauvegarde également le déplacement
				if (profondeur==infos->profondeur)
				{
					infos->meilleur_coup->pos_depart  = i;
					infos->meilleur_coup->pos_arrivee = possibilites_de_deplacement[j];
				}
			}
			
			j++;
		}
	}
	
	return alpha;
}



int IAJouer(void* data)
{
	void** thread_data=(void**) data;
	
	PPiece** echiquier=(PPiece**) thread_data[0];
	unsigned char* couleur_en_cours=(unsigned char*) (thread_data[1]);
	bool* IA_doit_finir_immediatement=(bool*)(thread_data[2]);
	bool* IA_vient_de_finir_de_jouer=(bool*)(thread_data[3]);
	bool* IA_faible_priorite=(bool*)(thread_data[4]);
	unsigned char IA_difficulte=*((unsigned char*) (thread_data[5]));
	unsigned int IA_temps=*((unsigned int*) (thread_data[6]));
	bool IA_ID=*((bool*) (thread_data[7]));
	bool* IA_abandonne=(bool*)(thread_data[8]);

	// avant de lancer le calcul de l'IA on attend que l'animation soit finie.
	// cela permet d'avoir une animation fluide et de plus on est sur que l'IA
	// ne jouera pas tant que l'animation n'est pas finie.
	SDL_Delay(1500);
	
	Uint32 temps_ecoule=SDL_GetTicks();
	
	InfoMinMax infos;
	CoupAJouer coupAJouer={99,99};
	int score;
	
	
	IA_difficulte+=5;
	int profondeur_base;
	if (IA_ID)
		profondeur_base=IA_difficulte-3;
	else
		profondeur_base=IA_difficulte-1;
	
	do
	{
		profondeur_base++;
		/*	tout d'abord on recrée un nouvel échiquier identique à celui sur
		lequel on est en train de jouer de manière à pouvoir travailler
		dessus sans toucher à l'original.
		*/
		PPiece* copieEchiquier[64];// l'échiquier est un tableau à deux dimensions de 8*8 cases
		for(int i=0;i<64;i++)
		{
			copieEchiquier[i]=new PPiece(*echiquier[i]);// recopie la pièce (fait appel au constructeur de copie)
			copieEchiquier[i]->setEchiquier(copieEchiquier);
		}
		
		piece_vide = new PPiece();
		
		infos.echiquier=copieEchiquier;
		infos.IA_doit_finir_immediatement=IA_doit_finir_immediatement;
		infos.couleur=*couleur_en_cours;
		infos.meilleur_coup=&coupAJouer;
		infos.faible_priorite=IA_faible_priorite;
		infos.compteur_pour_la_faible_priorite=0;
		infos.profondeur=profondeur_base;
		
		score=AlphaBeta(&infos,infos.profondeur,-INFINI,INFINI);
		
		/*	On n'oublie pas de supprimer l'échiquier qu'on a créé
			car c'est une copie de l'ancien dont on n'a plus besoin
		*/
		delete piece_vide;
		piece_vide=NULL;
		
		for(int i=0;i<64;i++)
		{
			delete copieEchiquier[i];
			copieEchiquier[i]=NULL;
		}
		
		temps_ecoule=SDL_GetTicks()-temps_ecoule;
	}
	while ((temps_ecoule<(1000*IA_temps)) && (profondeur_base<IA_difficulte));
	
	
	
	// si on a trouvé un coup à jouer
	if (coupAJouer.pos_depart<65 && coupAJouer.pos_arrivee<65)
	{
		// on n'abandonne pas
		*IA_abandonne=false;
		// et on joue notre coup
		echiquier[coupAJouer.pos_depart]->deplacer(coupAJouer.pos_arrivee%8,coupAJouer.pos_arrivee/8);
	}
	else // sinon on abandonne
	{
		*IA_abandonne=true;
		printf("%d\n",score);
	}
	
	

	// on prévient qu'on a terminé
	// NOTE: doit ****toujours**** être avant le return 0
	(*IA_vient_de_finir_de_jouer)=true;
	
	return 0;
}










