/**
 * @file main.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Fonction main, debut du jeu ici
 * Ce fichier contient également la documentation de la page principale.
 * La documentation a été écrite pour etre genérée par doxygen.
*/
//-------------------------------------------------------------------------------------------//
/**
 * @mainpage pouetChess : Jeu d'échec par Frédéric MARTIN utilisant l'algorithme MinMax
 * @section A-propos
 * Le programme a été développé sous Linux (Fedora Core 4) et 
 * écrit avec Kate (éditeur très puissant grace à ses nombreux plugins). 
 *
 * Il compile sans erreur ni warning avec gcc 4.0.x
 *
 * Attention à l'encodage des fichiers, ils sont en <b>UTF8</b>.
 * @section Introduction
 * pouetChess est un jeu d'échec en 3D qui utilise les bibliothèques suivantes :
 *
 * - SDL
 * - SDL_image
 * - OpenGL
 *
 * Grace a SDL le jeu est multi-platteforme (au moins Linux/Windows sont supportés).
 * SDL est une bibliothèque très utilisée pour la conception de jeux sous Linux.
 * A titre d'exemple, elle est utilisée par Quake 4 sous Linux !
 *
 * La configuration demandée pour pouvoir jouer est au <b>miminmum</b> de :
 *
 * - Processeur : 500Mhz
 * - Mémoire : 128 Mo
 * - Carte Graphique : c'est le point le plus important !
 * Une GeForce ou Radeon de moins de 5 ans avec 32 Mo de mémoire est vivement recommandé.
 *
*/


#include "scenes.h"
#include "scene_init.h"
#include "scene_menu_principal.h"
#include "scene_jeu_principal.h"

GlobalInfos* pGlobalInfos;

//-------------------------------------------------------------------//
//----------- Fonction main(): le programme commence ici ------------//
//-------------------------------------------------------------------//
int main(int argc, char *argv[])
{
	// la seule et unique instance de GlobalInfos est allouée ici
	pGlobalInfos = new GlobalInfos;
	if (pGlobalInfos->loadGameConfiguration()==false)
		return 1;
	
	// Initialisation de SDL
	if (SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		printf("SDL_Init(SDL_INIT_VIDEO) failed : %s\n",SDL_GetError());
		SDL_Quit();
		return 2;
	}
	
	SDL_WM_SetCaption("pouetChess version "VERSION_DU_JEU,"");
	
	// On obtient des informations vidéo qui seront utilisés pour connaitre la profondeur de couleur (bpp)
	const SDL_VideoInfo* info = NULL;
	info = SDL_GetVideoInfo();

	// Ceci ne devrait jamais arriver
	if (info == NULL) 
	{
		printf("SDL_GetVideoInfo() failed: %s\n",SDL_GetError());
		SDL_Quit();
		return 3;
	}
	
	
	// Création de la fenêtre
	// Essaye de créer en plein écran et si ça échoue essaye de créer juste une fenêtre
	if (pGlobalInfos->est_en_plein_ecran())
	{
		if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE | SDL_FULLSCREEN))
		{
			printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
			printf("I try the window mode\n");
			if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE))
			{
				printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
				SDL_Quit();
				return 4;
			}
			else
				printf("%dx%d in window mode is OK\n",pGlobalInfos->largeurEcran(),pGlobalInfos->hauteurEcran());
		}
	}
	else
	{
		if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE))
		{
			printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
			SDL_Quit();
			return 4;
		}
	}
	
	// On se prépare a afficher
	glClearColor(0.0f,0.0f,0.0f,1.0f);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// Initialisation du repère vue OpenGL
	glViewport (0, 0, (GLsizei)(pGlobalInfos->largeurEcran()), (GLsizei)(pGlobalInfos->hauteurEcran()));
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (45.0f, (GLfloat)(pGlobalInfos->largeurEcran())/(GLfloat)(pGlobalInfos->hauteurEcran()),
					1.0f, 10000.0f);		
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	
	// Initialisation du tampon de profondeur
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	
	// Efface l'écran
	SDL_GL_SwapBuffers();
	SDL_Delay(25);
	
	// Déclare et crée les scenes du jeu (mais ne charge rien)
	// on en a besoin pour le polymorphisme
	Echec_Scene* echec_scene[NOMBRE_DE_SCENES];
	
	echec_scene[0] = new E_scene_init;
	echec_scene[1] = new E_scene_intro;
	echec_scene[2] = new E_scene_menu_principal;
	echec_scene[3] = new E_scene_jeu_principal;
	
	// La premiere scène est SCENE_INIT.
	pGlobalInfos->changer_de_scene(E_SCENE_INIT);

	// Et c'est parti pour la boucle principale du jeu
	while( pGlobalInfos->le_jeu_est_actif() )
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Efface les tampons
		glLoadIdentity();											// Charge la matrice identité
		
		pGlobalInfos->actualiser();									// actualise toutes les infos globales
		echec_scene[pGlobalInfos->sceneActuelle()]->actualiser();	// actualise la scène actuelle
		
		SDL_GL_SwapBuffers();	// basculement de page: tout est affiché maintenant
	}
	
	// Le jeu est fini : on s'assure que toute la memoire allouée est bien restaurée
	for(int i=0;i<NOMBRE_DE_SCENES;i++)
	{
		echec_scene[i]->decharger();
		delete echec_scene[i];
	}
	
	//pGlobalInfos->sauverFichierDeConfiguration();
			
	delete pGlobalInfos;
	pGlobalInfos=NULL;
	
	SDL_Quit();
	return(0);
}

